/* IMPORT */
/* MAIN */
class Limiter {
    /* CONSTRUCTOR */
    constructor(options) {
        /* API */
        this.add = (fn) => {
            if (this.count < this.concurrency)
                return this.run(fn);
            return new Promise(resolve => {
                const callback = () => resolve(this.run(fn));
                this.queue.add(callback);
            });
        };
        this.flush = () => {
            for (const callback of this.queue) {
                if (this.count >= this.concurrency)
                    break;
                this.queue.delete(callback);
                callback();
            }
        };
        this.run = (fn) => {
            this.count += 1;
            const promise = fn();
            const cleanup = () => {
                this.count -= 1;
                this.flush();
            };
            promise.then(cleanup, cleanup);
            return promise;
        };
        this.concurrency = options.concurrency;
        this.count = 0;
        this.queue = new Set();
    }
}
/* EXPORT */
export default Limiter;
